# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .insights import (
    InsightsResource,
    AsyncInsightsResource,
    InsightsResourceWithRawResponse,
    AsyncInsightsResourceWithRawResponse,
    InsightsResourceWithStreamingResponse,
    AsyncInsightsResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.ai.conversations import (
    insight_group_update_params,
    insight_group_insight_groups_params,
    insight_group_retrieve_insight_groups_params,
)
from .....types.ai.conversations.insight_template_group_detail import InsightTemplateGroupDetail
from .....types.ai.conversations.insight_group_retrieve_insight_groups_response import (
    InsightGroupRetrieveInsightGroupsResponse,
)

__all__ = ["InsightGroupsResource", "AsyncInsightGroupsResource"]


class InsightGroupsResource(SyncAPIResource):
    @cached_property
    def insights(self) -> InsightsResource:
        return InsightsResource(self._client)

    @cached_property
    def with_raw_response(self) -> InsightGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return InsightGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InsightGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return InsightGroupsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InsightTemplateGroupDetail:
        """
        Get insight group by ID

        Args:
          group_id: The ID of the insight group

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return self._get(
            f"/ai/conversations/insight-groups/{group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InsightTemplateGroupDetail,
        )

    def update(
        self,
        group_id: str,
        *,
        description: str | Omit = omit,
        name: str | Omit = omit,
        webhook: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InsightTemplateGroupDetail:
        """
        Update an insight template group

        Args:
          group_id: The ID of the insight group

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return self._put(
            f"/ai/conversations/insight-groups/{group_id}",
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "webhook": webhook,
                },
                insight_group_update_params.InsightGroupUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InsightTemplateGroupDetail,
        )

    def delete(
        self,
        group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete insight group by ID

        Args:
          group_id: The ID of the insight group

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return self._delete(
            f"/ai/conversations/insight-groups/{group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def insight_groups(
        self,
        *,
        name: str,
        description: str | Omit = omit,
        webhook: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InsightTemplateGroupDetail:
        """
        Create a new insight group

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/ai/conversations/insight-groups",
            body=maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "webhook": webhook,
                },
                insight_group_insight_groups_params.InsightGroupInsightGroupsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InsightTemplateGroupDetail,
        )

    def retrieve_insight_groups(
        self,
        *,
        page: insight_group_retrieve_insight_groups_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InsightGroupRetrieveInsightGroupsResponse:
        """
        Get all insight groups

        Args:
          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/ai/conversations/insight-groups",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"page": page}, insight_group_retrieve_insight_groups_params.InsightGroupRetrieveInsightGroupsParams
                ),
            ),
            cast_to=InsightGroupRetrieveInsightGroupsResponse,
        )


class AsyncInsightGroupsResource(AsyncAPIResource):
    @cached_property
    def insights(self) -> AsyncInsightsResource:
        return AsyncInsightsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncInsightGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInsightGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInsightGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncInsightGroupsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InsightTemplateGroupDetail:
        """
        Get insight group by ID

        Args:
          group_id: The ID of the insight group

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return await self._get(
            f"/ai/conversations/insight-groups/{group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InsightTemplateGroupDetail,
        )

    async def update(
        self,
        group_id: str,
        *,
        description: str | Omit = omit,
        name: str | Omit = omit,
        webhook: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InsightTemplateGroupDetail:
        """
        Update an insight template group

        Args:
          group_id: The ID of the insight group

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return await self._put(
            f"/ai/conversations/insight-groups/{group_id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "webhook": webhook,
                },
                insight_group_update_params.InsightGroupUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InsightTemplateGroupDetail,
        )

    async def delete(
        self,
        group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete insight group by ID

        Args:
          group_id: The ID of the insight group

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return await self._delete(
            f"/ai/conversations/insight-groups/{group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def insight_groups(
        self,
        *,
        name: str,
        description: str | Omit = omit,
        webhook: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InsightTemplateGroupDetail:
        """
        Create a new insight group

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/ai/conversations/insight-groups",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "webhook": webhook,
                },
                insight_group_insight_groups_params.InsightGroupInsightGroupsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InsightTemplateGroupDetail,
        )

    async def retrieve_insight_groups(
        self,
        *,
        page: insight_group_retrieve_insight_groups_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InsightGroupRetrieveInsightGroupsResponse:
        """
        Get all insight groups

        Args:
          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/ai/conversations/insight-groups",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"page": page}, insight_group_retrieve_insight_groups_params.InsightGroupRetrieveInsightGroupsParams
                ),
            ),
            cast_to=InsightGroupRetrieveInsightGroupsResponse,
        )


class InsightGroupsResourceWithRawResponse:
    def __init__(self, insight_groups: InsightGroupsResource) -> None:
        self._insight_groups = insight_groups

        self.retrieve = to_raw_response_wrapper(
            insight_groups.retrieve,
        )
        self.update = to_raw_response_wrapper(
            insight_groups.update,
        )
        self.delete = to_raw_response_wrapper(
            insight_groups.delete,
        )
        self.insight_groups = to_raw_response_wrapper(
            insight_groups.insight_groups,
        )
        self.retrieve_insight_groups = to_raw_response_wrapper(
            insight_groups.retrieve_insight_groups,
        )

    @cached_property
    def insights(self) -> InsightsResourceWithRawResponse:
        return InsightsResourceWithRawResponse(self._insight_groups.insights)


class AsyncInsightGroupsResourceWithRawResponse:
    def __init__(self, insight_groups: AsyncInsightGroupsResource) -> None:
        self._insight_groups = insight_groups

        self.retrieve = async_to_raw_response_wrapper(
            insight_groups.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            insight_groups.update,
        )
        self.delete = async_to_raw_response_wrapper(
            insight_groups.delete,
        )
        self.insight_groups = async_to_raw_response_wrapper(
            insight_groups.insight_groups,
        )
        self.retrieve_insight_groups = async_to_raw_response_wrapper(
            insight_groups.retrieve_insight_groups,
        )

    @cached_property
    def insights(self) -> AsyncInsightsResourceWithRawResponse:
        return AsyncInsightsResourceWithRawResponse(self._insight_groups.insights)


class InsightGroupsResourceWithStreamingResponse:
    def __init__(self, insight_groups: InsightGroupsResource) -> None:
        self._insight_groups = insight_groups

        self.retrieve = to_streamed_response_wrapper(
            insight_groups.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            insight_groups.update,
        )
        self.delete = to_streamed_response_wrapper(
            insight_groups.delete,
        )
        self.insight_groups = to_streamed_response_wrapper(
            insight_groups.insight_groups,
        )
        self.retrieve_insight_groups = to_streamed_response_wrapper(
            insight_groups.retrieve_insight_groups,
        )

    @cached_property
    def insights(self) -> InsightsResourceWithStreamingResponse:
        return InsightsResourceWithStreamingResponse(self._insight_groups.insights)


class AsyncInsightGroupsResourceWithStreamingResponse:
    def __init__(self, insight_groups: AsyncInsightGroupsResource) -> None:
        self._insight_groups = insight_groups

        self.retrieve = async_to_streamed_response_wrapper(
            insight_groups.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            insight_groups.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            insight_groups.delete,
        )
        self.insight_groups = async_to_streamed_response_wrapper(
            insight_groups.insight_groups,
        )
        self.retrieve_insight_groups = async_to_streamed_response_wrapper(
            insight_groups.retrieve_insight_groups,
        )

    @cached_property
    def insights(self) -> AsyncInsightsResourceWithStreamingResponse:
        return AsyncInsightsResourceWithStreamingResponse(self._insight_groups.insights)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .voice_create_params import VoiceCreateParams as VoiceCreateParams
from .voice_list_response import VoiceListResponse as VoiceListResponse
from .voice_create_response import VoiceCreateResponse as VoiceCreateResponse
from .voice_delete_response import VoiceDeleteResponse as VoiceDeleteResponse
from .messaging_create_params import MessagingCreateParams as MessagingCreateParams
from .messaging_list_response import MessagingListResponse as MessagingListResponse
from .voice_retrieve_response import VoiceRetrieveResponse as VoiceRetrieveResponse
from .batch_csv_pagination_meta import BatchCsvPaginationMeta as BatchCsvPaginationMeta
from .cdr_detailed_req_response import CdrDetailedReqResponse as CdrDetailedReqResponse
from .messaging_create_response import MessagingCreateResponse as MessagingCreateResponse
from .messaging_delete_response import MessagingDeleteResponse as MessagingDeleteResponse
from .mdr_detail_report_response import MdrDetailReportResponse as MdrDetailReportResponse
from .stt_detail_report_response import SttDetailReportResponse as SttDetailReportResponse
from .messaging_retrieve_response import MessagingRetrieveResponse as MessagingRetrieveResponse
from .speech_to_text_create_params import SpeechToTextCreateParams as SpeechToTextCreateParams
from .speech_to_text_list_response import SpeechToTextListResponse as SpeechToTextListResponse
from .speech_to_text_create_response import SpeechToTextCreateResponse as SpeechToTextCreateResponse
from .speech_to_text_delete_response import SpeechToTextDeleteResponse as SpeechToTextDeleteResponse
from .voice_retrieve_fields_response import VoiceRetrieveFieldsResponse as VoiceRetrieveFieldsResponse
from .speech_to_text_retrieve_response import SpeechToTextRetrieveResponse as SpeechToTextRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["PortingPhoneNumberListParams", "Filter", "Page"]


class PortingPhoneNumberListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[porting_order_status]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """


class Filter(TypedDict, total=False):
    porting_order_status: Literal[
        "draft", "in-process", "submitted", "exception", "foc-date-confirmed", "cancel-pending", "ported", "cancelled"
    ]
    """Filter results by porting order status"""


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["ActionUpdateParams"]


class ActionUpdateParams(TypedDict, total=False):
    call_control_id: Required[str]
    """Unique identifier and token for controlling the call"""

    supervisor_role: Required[Literal["barge", "monitor", "none", "whisper"]]
    """Sets the participant as a supervisor for the conference.

    A conference can have multiple supervisors. "barge" means the supervisor enters
    the conference as a normal participant. This is the same as "none". "monitor"
    means the supervisor is muted but can hear all participants. "whisper" means
    that only the specified "whisper_call_control_ids" can hear the supervisor.
    Defaults to "none".
    """

    command_id: str
    """Use this field to avoid execution of duplicate commands.

    Telnyx will ignore subsequent commands with the same `command_id` as one that
    has already been executed.
    """

    whisper_call_control_ids: SequenceNotStr[str]
    """Array of unique call_control_ids the supervisor can whisper to.

    If none provided, the supervisor will join the conference as a monitoring
    participant only.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .call_update_params import CallUpdateParams as CallUpdateParams
from .call_initiate_params import CallInitiateParams as CallInitiateParams
from .call_update_response import CallUpdateResponse as CallUpdateResponse
from .call_initiate_response import CallInitiateResponse as CallInitiateResponse
from .texml_recording_subresources_uris import TexmlRecordingSubresourcesUris as TexmlRecordingSubresourcesUris
from .texml_get_call_recording_response_body import (
    TexmlGetCallRecordingResponseBody as TexmlGetCallRecordingResponseBody,
)
from .account_retrieve_recordings_json_params import (
    AccountRetrieveRecordingsJsonParams as AccountRetrieveRecordingsJsonParams,
)
from .account_retrieve_recordings_json_response import (
    AccountRetrieveRecordingsJsonResponse as AccountRetrieveRecordingsJsonResponse,
)
from .account_retrieve_transcriptions_json_params import (
    AccountRetrieveTranscriptionsJsonParams as AccountRetrieveTranscriptionsJsonParams,
)
from .account_retrieve_transcriptions_json_response import (
    AccountRetrieveTranscriptionsJsonResponse as AccountRetrieveTranscriptionsJsonResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo
from .conversation_channel_type import ConversationChannelType

__all__ = ["ScheduledEventCreateParams"]


class ScheduledEventCreateParams(TypedDict, total=False):
    scheduled_at_fixed_datetime: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """The datetime at which the event should be scheduled. Formatted as ISO 8601."""

    telnyx_agent_target: Required[str]
    """The phone number, SIP URI, to schedule the call or text from."""

    telnyx_conversation_channel: Required[ConversationChannelType]

    telnyx_end_user_target: Required[str]
    """The phone number, SIP URI, to schedule the call or text to."""

    conversation_metadata: Dict[str, Union[str, int, bool]]
    """Metadata associated with the conversation.

    Telnyx provides several pieces of metadata, but customers can also add their
    own.
    """

    text: str
    """Required for sms scheduled events. The text to be sent to the end user."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .pagination_meta import PaginationMeta
from .private_wireless_gateway import PrivateWirelessGateway

__all__ = ["PrivateWirelessGatewayListResponse"]


class PrivateWirelessGatewayListResponse(BaseModel):
    data: Optional[List[PrivateWirelessGateway]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["MigrationCreateParams"]


class MigrationCreateParams(TypedDict, total=False):
    source_id: Required[str]
    """ID of the Migration Source from which to migrate data."""

    target_bucket_name: Required[str]
    """Bucket name to migrate the data into.

    Will default to the same name as the `source_bucket_name`.
    """

    target_region: Required[str]
    """Telnyx Cloud Storage region to migrate the data to."""

    refresh: bool
    """
    If true, will continue to poll the source bucket to ensure new data is
    continually migrated over.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.campaign_builder.brand_qualify_by_usecase_response import BrandQualifyByUsecaseResponse

__all__ = ["BrandResource", "AsyncBrandResource"]


class BrandResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BrandResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return BrandResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BrandResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return BrandResourceWithStreamingResponse(self)

    def qualify_by_usecase(
        self,
        usecase: str,
        *,
        brand_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandQualifyByUsecaseResponse:
        """
        This endpoint allows you to see whether or not the supplied brand is suitable
        for your desired campaign use case.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        if not usecase:
            raise ValueError(f"Expected a non-empty value for `usecase` but received {usecase!r}")
        return self._get(
            f"/campaignBuilder/brand/{brand_id}/usecase/{usecase}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandQualifyByUsecaseResponse,
        )


class AsyncBrandResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBrandResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBrandResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBrandResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncBrandResourceWithStreamingResponse(self)

    async def qualify_by_usecase(
        self,
        usecase: str,
        *,
        brand_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandQualifyByUsecaseResponse:
        """
        This endpoint allows you to see whether or not the supplied brand is suitable
        for your desired campaign use case.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        if not usecase:
            raise ValueError(f"Expected a non-empty value for `usecase` but received {usecase!r}")
        return await self._get(
            f"/campaignBuilder/brand/{brand_id}/usecase/{usecase}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandQualifyByUsecaseResponse,
        )


class BrandResourceWithRawResponse:
    def __init__(self, brand: BrandResource) -> None:
        self._brand = brand

        self.qualify_by_usecase = to_raw_response_wrapper(
            brand.qualify_by_usecase,
        )


class AsyncBrandResourceWithRawResponse:
    def __init__(self, brand: AsyncBrandResource) -> None:
        self._brand = brand

        self.qualify_by_usecase = async_to_raw_response_wrapper(
            brand.qualify_by_usecase,
        )


class BrandResourceWithStreamingResponse:
    def __init__(self, brand: BrandResource) -> None:
        self._brand = brand

        self.qualify_by_usecase = to_streamed_response_wrapper(
            brand.qualify_by_usecase,
        )


class AsyncBrandResourceWithStreamingResponse:
    def __init__(self, brand: AsyncBrandResource) -> None:
        self._brand = brand

        self.qualify_by_usecase = async_to_streamed_response_wrapper(
            brand.qualify_by_usecase,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from ..shared.porting_order_status import PortingOrderStatus

__all__ = [
    "EventRetrieveResponse",
    "Data",
    "DataPayload",
    "DataPayloadWebhookPortingOrderDeletedPayload",
    "DataPayloadWebhookPortingOrderMessagingChangedPayload",
    "DataPayloadWebhookPortingOrderMessagingChangedPayloadMessaging",
    "DataPayloadWebhookPortingOrderStatusChangedPayload",
    "DataPayloadWebhookPortingOrderNewCommentPayload",
    "DataPayloadWebhookPortingOrderNewCommentPayloadComment",
    "DataPayloadWebhookPortingOrderSplitPayload",
    "DataPayloadWebhookPortingOrderSplitPayloadFrom",
    "DataPayloadWebhookPortingOrderSplitPayloadPortingPhoneNumber",
    "DataPayloadWebhookPortingOrderSplitPayloadTo",
]


class DataPayloadWebhookPortingOrderDeletedPayload(BaseModel):
    id: Optional[str] = None
    """Identifies the porting order that was deleted."""

    customer_reference: Optional[str] = None
    """Identifies the customer reference associated with the porting order."""

    deleted_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the porting order was deleted."""


class DataPayloadWebhookPortingOrderMessagingChangedPayloadMessaging(BaseModel):
    enable_messaging: Optional[bool] = None
    """
    Indicates whether Telnyx will port messaging capabilities from the losing
    carrier. If false, any messaging capabilities will stay with their current
    provider.
    """

    messaging_capable: Optional[bool] = None
    """Indicates whether the porting order is messaging capable."""

    messaging_port_completed: Optional[bool] = None
    """Indicates whether the messaging port is completed."""

    messaging_port_status: Optional[
        Literal["not_applicable", "pending", "activating", "exception", "canceled", "partial_port_complete", "ported"]
    ] = None
    """Indicates the messaging port status of the porting order."""


class DataPayloadWebhookPortingOrderMessagingChangedPayload(BaseModel):
    id: Optional[str] = None
    """Identifies the porting order that was moved."""

    customer_reference: Optional[str] = None
    """Identifies the customer reference associated with the porting order."""

    messaging: Optional[DataPayloadWebhookPortingOrderMessagingChangedPayloadMessaging] = None
    """The messaging portability status of the porting order."""

    support_key: Optional[str] = None
    """Identifies the support key associated with the porting order."""


class DataPayloadWebhookPortingOrderStatusChangedPayload(BaseModel):
    id: Optional[str] = None
    """Identifies the porting order that was moved."""

    customer_reference: Optional[str] = None
    """Identifies the customer reference associated with the porting order."""

    status: Optional[PortingOrderStatus] = None
    """Porting order status"""

    support_key: Optional[str] = None
    """Identifies the support key associated with the porting order."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the porting order was moved."""

    webhook_url: Optional[str] = None
    """The URL to send the webhook to."""


class DataPayloadWebhookPortingOrderNewCommentPayloadComment(BaseModel):
    id: Optional[str] = None
    """Identifies the comment."""

    body: Optional[str] = None
    """The body of the comment."""

    inserted_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the comment was created."""

    user_id: Optional[str] = None
    """Identifies the user that create the comment."""

    user_type: Optional[Literal["user", "admin", "system"]] = None
    """Identifies the type of the user that created the comment."""


class DataPayloadWebhookPortingOrderNewCommentPayload(BaseModel):
    comment: Optional[DataPayloadWebhookPortingOrderNewCommentPayloadComment] = None
    """The comment that was added to the porting order."""

    porting_order_id: Optional[str] = None
    """Identifies the porting order that the comment was added to."""

    support_key: Optional[str] = None
    """Identifies the support key associated with the porting order."""


class DataPayloadWebhookPortingOrderSplitPayloadFrom(BaseModel):
    id: Optional[str] = None
    """Identifies the porting order that was split."""


class DataPayloadWebhookPortingOrderSplitPayloadPortingPhoneNumber(BaseModel):
    id: Optional[str] = None
    """Identifies the porting phone number that was moved."""


class DataPayloadWebhookPortingOrderSplitPayloadTo(BaseModel):
    id: Optional[str] = None
    """Identifies the porting order that was split."""


class DataPayloadWebhookPortingOrderSplitPayload(BaseModel):
    from_: Optional[DataPayloadWebhookPortingOrderSplitPayloadFrom] = FieldInfo(alias="from", default=None)
    """The porting order that was split."""

    porting_phone_numbers: Optional[List[DataPayloadWebhookPortingOrderSplitPayloadPortingPhoneNumber]] = None
    """The list of porting phone numbers that were moved to the new porting order."""

    to: Optional[DataPayloadWebhookPortingOrderSplitPayloadTo] = None
    """The new porting order that the phone numbers was moved to."""


DataPayload: TypeAlias = Union[
    DataPayloadWebhookPortingOrderDeletedPayload,
    DataPayloadWebhookPortingOrderMessagingChangedPayload,
    DataPayloadWebhookPortingOrderStatusChangedPayload,
    DataPayloadWebhookPortingOrderNewCommentPayload,
    DataPayloadWebhookPortingOrderSplitPayload,
]


class Data(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the event."""

    available_notification_methods: Optional[List[Literal["email", "webhook", "webhook_v1"]]] = None
    """Indicates the notification methods used."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    event_type: Optional[
        Literal[
            "porting_order.deleted",
            "porting_order.loa_updated",
            "porting_order.messaging_changed",
            "porting_order.status_changed",
            "porting_order.sharing_token_expired",
            "porting_order.new_comment",
            "porting_order.split",
        ]
    ] = None
    """Identifies the event type"""

    payload: Optional[DataPayload] = None
    """The webhook payload for the porting_order.deleted event"""

    payload_status: Optional[Literal["created", "completed"]] = None
    """The status of the payload generation."""

    porting_order_id: Optional[str] = None
    """Identifies the porting order associated with the event."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""


class EventRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    SimCardDataUsageNotificationListResponse,
    SimCardDataUsageNotificationCreateResponse,
    SimCardDataUsageNotificationDeleteResponse,
    SimCardDataUsageNotificationUpdateResponse,
    SimCardDataUsageNotificationRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSimCardDataUsageNotifications:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        sim_card_data_usage_notification = client.sim_card_data_usage_notifications.create(
            sim_card_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            threshold={},
        )
        assert_matches_type(
            SimCardDataUsageNotificationCreateResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        sim_card_data_usage_notification = client.sim_card_data_usage_notifications.create(
            sim_card_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            threshold={
                "amount": "2048.1",
                "unit": "MB",
            },
        )
        assert_matches_type(
            SimCardDataUsageNotificationCreateResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.sim_card_data_usage_notifications.with_raw_response.create(
            sim_card_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            threshold={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_data_usage_notification = response.parse()
        assert_matches_type(
            SimCardDataUsageNotificationCreateResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.sim_card_data_usage_notifications.with_streaming_response.create(
            sim_card_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            threshold={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_data_usage_notification = response.parse()
            assert_matches_type(
                SimCardDataUsageNotificationCreateResponse, sim_card_data_usage_notification, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        sim_card_data_usage_notification = client.sim_card_data_usage_notifications.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(
            SimCardDataUsageNotificationRetrieveResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.sim_card_data_usage_notifications.with_raw_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_data_usage_notification = response.parse()
        assert_matches_type(
            SimCardDataUsageNotificationRetrieveResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.sim_card_data_usage_notifications.with_streaming_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_data_usage_notification = response.parse()
            assert_matches_type(
                SimCardDataUsageNotificationRetrieveResponse, sim_card_data_usage_notification, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_card_data_usage_notifications.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        sim_card_data_usage_notification = client.sim_card_data_usage_notifications.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(
            SimCardDataUsageNotificationUpdateResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        sim_card_data_usage_notification = client.sim_card_data_usage_notifications.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            sim_card_id="b34c1683-cd85-4493-b9a5-315eb4bc5e19",
            threshold={
                "amount": "2048.0",
                "unit": "MB",
            },
        )
        assert_matches_type(
            SimCardDataUsageNotificationUpdateResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.sim_card_data_usage_notifications.with_raw_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_data_usage_notification = response.parse()
        assert_matches_type(
            SimCardDataUsageNotificationUpdateResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.sim_card_data_usage_notifications.with_streaming_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_data_usage_notification = response.parse()
            assert_matches_type(
                SimCardDataUsageNotificationUpdateResponse, sim_card_data_usage_notification, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_card_data_usage_notifications.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        sim_card_data_usage_notification = client.sim_card_data_usage_notifications.list()
        assert_matches_type(
            SimCardDataUsageNotificationListResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        sim_card_data_usage_notification = client.sim_card_data_usage_notifications.list(
            filter_sim_card_id="47a1c2b0-cc7b-4ab1-bb98-b33fb0fc61b9",
            page_number=1,
            page_size=1,
        )
        assert_matches_type(
            SimCardDataUsageNotificationListResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.sim_card_data_usage_notifications.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_data_usage_notification = response.parse()
        assert_matches_type(
            SimCardDataUsageNotificationListResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.sim_card_data_usage_notifications.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_data_usage_notification = response.parse()
            assert_matches_type(
                SimCardDataUsageNotificationListResponse, sim_card_data_usage_notification, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        sim_card_data_usage_notification = client.sim_card_data_usage_notifications.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(
            SimCardDataUsageNotificationDeleteResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.sim_card_data_usage_notifications.with_raw_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_data_usage_notification = response.parse()
        assert_matches_type(
            SimCardDataUsageNotificationDeleteResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.sim_card_data_usage_notifications.with_streaming_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_data_usage_notification = response.parse()
            assert_matches_type(
                SimCardDataUsageNotificationDeleteResponse, sim_card_data_usage_notification, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_card_data_usage_notifications.with_raw_response.delete(
                "",
            )


class TestAsyncSimCardDataUsageNotifications:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        sim_card_data_usage_notification = await async_client.sim_card_data_usage_notifications.create(
            sim_card_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            threshold={},
        )
        assert_matches_type(
            SimCardDataUsageNotificationCreateResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        sim_card_data_usage_notification = await async_client.sim_card_data_usage_notifications.create(
            sim_card_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            threshold={
                "amount": "2048.1",
                "unit": "MB",
            },
        )
        assert_matches_type(
            SimCardDataUsageNotificationCreateResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_data_usage_notifications.with_raw_response.create(
            sim_card_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            threshold={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_data_usage_notification = await response.parse()
        assert_matches_type(
            SimCardDataUsageNotificationCreateResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_data_usage_notifications.with_streaming_response.create(
            sim_card_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            threshold={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_data_usage_notification = await response.parse()
            assert_matches_type(
                SimCardDataUsageNotificationCreateResponse, sim_card_data_usage_notification, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        sim_card_data_usage_notification = await async_client.sim_card_data_usage_notifications.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(
            SimCardDataUsageNotificationRetrieveResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_data_usage_notifications.with_raw_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_data_usage_notification = await response.parse()
        assert_matches_type(
            SimCardDataUsageNotificationRetrieveResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_data_usage_notifications.with_streaming_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_data_usage_notification = await response.parse()
            assert_matches_type(
                SimCardDataUsageNotificationRetrieveResponse, sim_card_data_usage_notification, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_card_data_usage_notifications.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        sim_card_data_usage_notification = await async_client.sim_card_data_usage_notifications.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(
            SimCardDataUsageNotificationUpdateResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        sim_card_data_usage_notification = await async_client.sim_card_data_usage_notifications.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            sim_card_id="b34c1683-cd85-4493-b9a5-315eb4bc5e19",
            threshold={
                "amount": "2048.0",
                "unit": "MB",
            },
        )
        assert_matches_type(
            SimCardDataUsageNotificationUpdateResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_data_usage_notifications.with_raw_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_data_usage_notification = await response.parse()
        assert_matches_type(
            SimCardDataUsageNotificationUpdateResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_data_usage_notifications.with_streaming_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_data_usage_notification = await response.parse()
            assert_matches_type(
                SimCardDataUsageNotificationUpdateResponse, sim_card_data_usage_notification, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_card_data_usage_notifications.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        sim_card_data_usage_notification = await async_client.sim_card_data_usage_notifications.list()
        assert_matches_type(
            SimCardDataUsageNotificationListResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        sim_card_data_usage_notification = await async_client.sim_card_data_usage_notifications.list(
            filter_sim_card_id="47a1c2b0-cc7b-4ab1-bb98-b33fb0fc61b9",
            page_number=1,
            page_size=1,
        )
        assert_matches_type(
            SimCardDataUsageNotificationListResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_data_usage_notifications.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_data_usage_notification = await response.parse()
        assert_matches_type(
            SimCardDataUsageNotificationListResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_data_usage_notifications.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_data_usage_notification = await response.parse()
            assert_matches_type(
                SimCardDataUsageNotificationListResponse, sim_card_data_usage_notification, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        sim_card_data_usage_notification = await async_client.sim_card_data_usage_notifications.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(
            SimCardDataUsageNotificationDeleteResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_data_usage_notifications.with_raw_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_data_usage_notification = await response.parse()
        assert_matches_type(
            SimCardDataUsageNotificationDeleteResponse, sim_card_data_usage_notification, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_data_usage_notifications.with_streaming_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_data_usage_notification = await response.parse()
            assert_matches_type(
                SimCardDataUsageNotificationDeleteResponse, sim_card_data_usage_notification, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_card_data_usage_notifications.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["DocReqsRequirementType", "AcceptanceCriteria"]


class AcceptanceCriteria(BaseModel):
    acceptable_characters: Optional[str] = None
    """Specifies the allowed characters as a string"""

    acceptable_values: Optional[List[str]] = None
    """Specifies the list of strictly possible values for the requirement.

    Ignored when empty
    """

    locality_limit: Optional[str] = None
    """Specifies geography-based acceptance criteria"""

    max_length: Optional[int] = None
    """Maximum length allowed for the value"""

    min_length: Optional[int] = None
    """Minimum length allowed for the value"""

    time_limit: Optional[str] = None
    """Specifies time-based acceptance criteria"""


class DocReqsRequirementType(BaseModel):
    id: Optional[str] = None
    """Identifies the associated document"""

    acceptance_criteria: Optional[AcceptanceCriteria] = None
    """Specifies objective criteria for acceptance"""

    created_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was created."""

    description: Optional[str] = None
    """Describes the requirement type"""

    example: Optional[str] = None
    """Provides one or more examples of acceptable documents"""

    name: Optional[str] = None
    """A short descriptive name for this requirement_type"""

    record_type: Optional[str] = None
    """Identifies the type of the resource"""

    type: Optional[Literal["document", "address", "textual"]] = None
    """Defines the type of this requirement type"""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["AuthenticationProviderListParams", "Page"]


class AuthenticationProviderListParams(TypedDict, total=False):
    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[number], page[size]
    """

    sort: Literal[
        "name",
        "-name",
        "short_name",
        "-short_name",
        "active",
        "-active",
        "created_at",
        "-created_at",
        "updated_at",
        "-updated_at",
    ]
    """Specifies the sort order for results.

    By default sorting direction is ascending. To have the results sorted in
    descending order add the <code>-</code> prefix.<br/><br/> That is: <ul>

      <li>
        <code>name</code>: sorts the result by the
        <code>name</code> field in ascending order.
      </li>
      <li>
        <code>-name</code>: sorts the result by the
        <code>name</code> field in descending order.
      </li>
    </ul><br/>If not given, results are sorted by <code>created_at</code> in descending order.
    """


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr
from .google_transcription_language import GoogleTranscriptionLanguage
from .telnyx_transcription_language import TelnyxTranscriptionLanguage
from .transcription_engine_a_config_param import TranscriptionEngineAConfigParam
from .transcription_engine_b_config_param import TranscriptionEngineBConfigParam

__all__ = [
    "ActionStartTranscriptionParams",
    "TranscriptionEngineConfig",
    "TranscriptionEngineConfigTranscriptionEngineGoogleConfig",
    "TranscriptionEngineConfigTranscriptionEngineGoogleConfigSpeechContext",
    "TranscriptionEngineConfigTranscriptionEngineTelnyxConfig",
    "TranscriptionEngineConfigTranscriptionEngineDeepgramConfig",
]


class ActionStartTranscriptionParams(TypedDict, total=False):
    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    transcription_engine: Literal["Google", "Telnyx", "Deepgram", "A", "B"]
    """Engine to use for speech recognition.

    Legacy values `A` - `Google`, `B` - `Telnyx` are supported for backward
    compatibility.
    """

    transcription_engine_config: TranscriptionEngineConfig

    transcription_tracks: str
    """Indicates which leg of the call will be transcribed.

    Use `inbound` for the leg that requested the transcription, `outbound` for the
    other leg, and `both` for both legs of the call. Will default to `inbound`.
    """


class TranscriptionEngineConfigTranscriptionEngineGoogleConfigSpeechContext(TypedDict, total=False):
    boost: float
    """Boost factor for the speech context."""

    phrases: SequenceNotStr[str]


class TranscriptionEngineConfigTranscriptionEngineGoogleConfig(TypedDict, total=False):
    enable_speaker_diarization: bool
    """Enables speaker diarization."""

    hints: SequenceNotStr[str]
    """Hints to improve transcription accuracy."""

    interim_results: bool
    """Whether to send also interim results.

    If set to false, only final results will be sent.
    """

    language: GoogleTranscriptionLanguage
    """Language to use for speech recognition"""

    max_speaker_count: int
    """Defines maximum number of speakers in the conversation."""

    min_speaker_count: int
    """Defines minimum number of speakers in the conversation."""

    model: Literal[
        "latest_long",
        "latest_short",
        "command_and_search",
        "phone_call",
        "video",
        "default",
        "medical_conversation",
        "medical_dictation",
    ]
    """The model to use for transcription."""

    profanity_filter: bool
    """Enables profanity_filter."""

    speech_context: Iterable[TranscriptionEngineConfigTranscriptionEngineGoogleConfigSpeechContext]
    """Speech context to improve transcription accuracy."""

    transcription_engine: Literal["Google"]
    """Engine identifier for Google transcription service"""

    use_enhanced: bool
    """Enables enhanced transcription, this works for models `phone_call` and `video`."""


class TranscriptionEngineConfigTranscriptionEngineTelnyxConfig(TypedDict, total=False):
    language: TelnyxTranscriptionLanguage
    """Language to use for speech recognition"""

    transcription_engine: Literal["Telnyx"]
    """Engine identifier for Telnyx transcription service"""

    transcription_model: Literal["openai/whisper-tiny", "openai/whisper-large-v3-turbo"]
    """The model to use for transcription."""


class TranscriptionEngineConfigTranscriptionEngineDeepgramConfig(TypedDict, total=False):
    transcription_engine: Required[Literal["Deepgram"]]
    """Engine identifier for Deepgram transcription service"""

    transcription_model: Required[Literal["deepgram/nova-2", "deepgram/nova-3"]]
    """The model to use for transcription."""

    language: Literal[
        "bg",
        "ca",
        "zh",
        "zh-CN",
        "zh-Hans",
        "zh-TW",
        "zh-Hant",
        "zh-HK",
        "cs",
        "da",
        "da-DK",
        "nl",
        "en",
        "en-US",
        "en-AU",
        "en-GB",
        "en-NZ",
        "en-IN",
        "et",
        "fi",
        "nl-BE",
        "fr",
        "fr-CA",
        "de",
        "de-CH",
        "el",
        "hi",
        "hu",
        "id",
        "it",
        "ja",
        "ko",
        "ko-KR",
        "lv",
        "lt",
        "ms",
        "no",
        "pl",
        "pt",
        "pt-BR",
        "pt-PT",
        "ro",
        "ru",
        "sk",
        "es",
        "es-419",
        "sv",
        "sv-SE",
        "th",
        "th-TH",
        "tr",
        "uk",
        "vi",
        "auto_detect",
    ]
    """Language to use for speech recognition.

    Available languages depend on the selected model.
    """


TranscriptionEngineConfig: TypeAlias = Union[
    TranscriptionEngineConfigTranscriptionEngineGoogleConfig,
    TranscriptionEngineConfigTranscriptionEngineTelnyxConfig,
    TranscriptionEngineConfigTranscriptionEngineDeepgramConfig,
    TranscriptionEngineAConfigParam,
    TranscriptionEngineBConfigParam,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .access_ip_range import AccessIPRange
from .pagination_meta_cloudflare_ip_list_sync import PaginationMetaCloudflareIPListSync

__all__ = ["AccessIPRangeListResponse"]


class AccessIPRangeListResponse(BaseModel):
    data: List[AccessIPRange]

    meta: PaginationMetaCloudflareIPListSync



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["InsightTemplate"]


class InsightTemplate(BaseModel):
    id: str

    created_at: datetime

    instructions: str

    insight_type: Optional[Literal["custom", "default"]] = None

    json_schema: Union[str, object, None] = None
    """If specified, the output will follow the JSON schema."""

    name: Optional[str] = None

    webhook: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["TelephonyCredential"]


class TelephonyCredential(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    created_at: Optional[str] = None
    """ISO-8601 formatted date indicating when the resource was created."""

    expired: Optional[bool] = None
    """Defaults to false"""

    expires_at: Optional[str] = None
    """ISO-8601 formatted date indicating when the resource will expire."""

    name: Optional[str] = None

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    resource_id: Optional[str] = None
    """Identifies the resource this credential is associated with."""

    sip_password: Optional[str] = None
    """The randomly generated SIP password for the credential."""

    sip_username: Optional[str] = None
    """The randomly generated SIP username for the credential."""

    updated_at: Optional[str] = None
    """ISO-8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from ..._types import SequenceNotStr

__all__ = ["BillingBundleListParams", "Filter", "Page"]


class BillingBundleListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Supports filtering by country_iso and resource. Examples: filter[country_iso]=US
    or filter[resource]=+15617819942
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """

    authorization_bearer: str
    """Authenticates the request with your Telnyx API V2 KEY"""


class Filter(TypedDict, total=False):
    country_iso: SequenceNotStr[str]
    """Filter by country code."""

    resource: SequenceNotStr[str]
    """Filter by resource."""


class Page(TypedDict, total=False):
    number: int
    """The page number to load."""

    size: int
    """The size of the page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .pagination_meta import PaginationMeta
from .available_service import AvailableService

__all__ = ["NetworkCoverageListResponse", "Data", "DataLocation"]


class DataLocation(BaseModel):
    code: Optional[str] = None
    """Location code."""

    name: Optional[str] = None
    """Human readable name of location."""

    pop: Optional[str] = None
    """Point of presence of location."""

    region: Optional[str] = None
    """Identifies the geographical region of location."""

    site: Optional[str] = None
    """Site of location."""


class Data(BaseModel):
    available_services: Optional[List[AvailableService]] = None
    """List of interface types supported in this region."""

    location: Optional[DataLocation] = None

    record_type: Optional[str] = None
    """Identifies the type of the resource."""


class NetworkCoverageListResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Required, Annotated, TypedDict

from .volume import Volume
from ...._utils import PropertyInfo
from .url_param import URLParam
from .use_case_categories import UseCaseCategories
from .tf_phone_number_param import TfPhoneNumberParam
from .toll_free_verification_entity_type import TollFreeVerificationEntityType

__all__ = ["RequestCreateParams"]


class RequestCreateParams(TypedDict, total=False):
    additional_information: Required[Annotated[str, PropertyInfo(alias="additionalInformation")]]
    """Any additional information"""

    business_addr1: Required[Annotated[str, PropertyInfo(alias="businessAddr1")]]
    """Line 1 of the business address"""

    business_city: Required[Annotated[str, PropertyInfo(alias="businessCity")]]
    """The city of the business address; the first letter should be capitalized"""

    business_contact_email: Required[Annotated[str, PropertyInfo(alias="businessContactEmail")]]
    """The email address of the business contact"""

    business_contact_first_name: Required[Annotated[str, PropertyInfo(alias="businessContactFirstName")]]
    """
    First name of the business contact; there are no specific requirements on
    formatting
    """

    business_contact_last_name: Required[Annotated[str, PropertyInfo(alias="businessContactLastName")]]
    """
    Last name of the business contact; there are no specific requirements on
    formatting
    """

    business_contact_phone: Required[Annotated[str, PropertyInfo(alias="businessContactPhone")]]
    """The phone number of the business contact in E.164 format"""

    business_name: Required[Annotated[str, PropertyInfo(alias="businessName")]]
    """Name of the business; there are no specific formatting requirements"""

    business_state: Required[Annotated[str, PropertyInfo(alias="businessState")]]
    """
    The full name of the state (not the 2 letter code) of the business address; the
    first letter should be capitalized
    """

    business_zip: Required[Annotated[str, PropertyInfo(alias="businessZip")]]
    """The ZIP code of the business address"""

    corporate_website: Required[Annotated[str, PropertyInfo(alias="corporateWebsite")]]
    """A URL, including the scheme, pointing to the corporate website"""

    isv_reseller: Required[Annotated[str, PropertyInfo(alias="isvReseller")]]
    """ISV name"""

    message_volume: Required[Annotated[Volume, PropertyInfo(alias="messageVolume")]]
    """Message Volume Enums"""

    opt_in_workflow: Required[Annotated[str, PropertyInfo(alias="optInWorkflow")]]
    """
    Human-readable description of how end users will opt into receiving messages
    from the given phone numbers
    """

    opt_in_workflow_image_urls: Required[Annotated[Iterable[URLParam], PropertyInfo(alias="optInWorkflowImageURLs")]]
    """Images showing the opt-in workflow"""

    phone_numbers: Required[Annotated[Iterable[TfPhoneNumberParam], PropertyInfo(alias="phoneNumbers")]]
    """The phone numbers to request the verification of"""

    production_message_content: Required[Annotated[str, PropertyInfo(alias="productionMessageContent")]]
    """An example of a message that will be sent from the given phone numbers"""

    use_case: Required[Annotated[UseCaseCategories, PropertyInfo(alias="useCase")]]
    """Tollfree usecase categories"""

    use_case_summary: Required[Annotated[str, PropertyInfo(alias="useCaseSummary")]]
    """Human-readable summary of the desired use-case"""

    age_gated_content: Annotated[bool, PropertyInfo(alias="ageGatedContent")]
    """Indicates if messaging content requires age gating (e.g., 18+).

    Defaults to false if not provided.
    """

    business_addr2: Annotated[str, PropertyInfo(alias="businessAddr2")]
    """Line 2 of the business address"""

    business_registration_country: Annotated[Optional[str], PropertyInfo(alias="businessRegistrationCountry")]
    """ISO 3166-1 alpha-2 country code of the issuing business authority.

    Must be exactly 2 letters. Automatically converted to uppercase. Required from
    January 2026.
    """

    business_registration_number: Annotated[Optional[str], PropertyInfo(alias="businessRegistrationNumber")]
    """
    Official business registration number (e.g., Employer Identification Number
    (EIN) in the U.S.). Required from January 2026.
    """

    business_registration_type: Annotated[Optional[str], PropertyInfo(alias="businessRegistrationType")]
    """Type of business registration being provided. Required from January 2026."""

    doing_business_as: Annotated[Optional[str], PropertyInfo(alias="doingBusinessAs")]
    """Doing Business As (DBA) name if different from legal name"""

    entity_type: Annotated[Optional[TollFreeVerificationEntityType], PropertyInfo(alias="entityType")]
    """Business entity classification"""

    help_message_response: Annotated[Optional[str], PropertyInfo(alias="helpMessageResponse")]
    """The message returned when users text 'HELP'"""

    opt_in_confirmation_response: Annotated[Optional[str], PropertyInfo(alias="optInConfirmationResponse")]
    """Message sent to users confirming their opt-in to receive messages"""

    opt_in_keywords: Annotated[Optional[str], PropertyInfo(alias="optInKeywords")]
    """Keywords used to collect and process consumer opt-ins"""

    privacy_policy_url: Annotated[Optional[str], PropertyInfo(alias="privacyPolicyURL")]
    """URL pointing to the business's privacy policy.

    Plain string, no URL format validation.
    """

    terms_and_condition_url: Annotated[Optional[str], PropertyInfo(alias="termsAndConditionURL")]
    """URL pointing to the business's terms and conditions.

    Plain string, no URL format validation.
    """

    webhook_url: Annotated[str, PropertyInfo(alias="webhookUrl")]
    """URL that should receive webhooks relating to this verification request"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["AgentUpdateParams"]


class AgentUpdateParams(TypedDict, total=False):
    profile_id: Optional[str]
    """Messaging profile ID associated with the RCS Agent"""

    webhook_failover_url: Optional[str]
    """Failover URL to receive RCS events"""

    webhook_url: Optional[str]
    """URL to receive RCS events"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .external_connection import ExternalConnection
from .external_voice_integrations_pagination_meta import ExternalVoiceIntegrationsPaginationMeta

__all__ = ["ExternalConnectionListResponse"]


class ExternalConnectionListResponse(BaseModel):
    data: Optional[List[ExternalConnection]] = None

    meta: Optional[ExternalVoiceIntegrationsPaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from .calls.calls import (
    CallsResource,
    AsyncCallsResource,
    CallsResourceWithRawResponse,
    AsyncCallsResourceWithRawResponse,
    CallsResourceWithStreamingResponse,
    AsyncCallsResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....types.texml import account_retrieve_recordings_json_params, account_retrieve_transcriptions_json_params
from ...._base_client import make_request_options
from .recordings.recordings import (
    RecordingsResource,
    AsyncRecordingsResource,
    RecordingsResourceWithRawResponse,
    AsyncRecordingsResourceWithRawResponse,
    RecordingsResourceWithStreamingResponse,
    AsyncRecordingsResourceWithStreamingResponse,
)
from .conferences.conferences import (
    ConferencesResource,
    AsyncConferencesResource,
    ConferencesResourceWithRawResponse,
    AsyncConferencesResourceWithRawResponse,
    ConferencesResourceWithStreamingResponse,
    AsyncConferencesResourceWithStreamingResponse,
)
from .transcriptions.transcriptions import (
    TranscriptionsResource,
    AsyncTranscriptionsResource,
    TranscriptionsResourceWithRawResponse,
    AsyncTranscriptionsResourceWithRawResponse,
    TranscriptionsResourceWithStreamingResponse,
    AsyncTranscriptionsResourceWithStreamingResponse,
)
from ....types.texml.account_retrieve_recordings_json_response import AccountRetrieveRecordingsJsonResponse
from ....types.texml.account_retrieve_transcriptions_json_response import AccountRetrieveTranscriptionsJsonResponse

__all__ = ["AccountsResource", "AsyncAccountsResource"]


class AccountsResource(SyncAPIResource):
    @cached_property
    def calls(self) -> CallsResource:
        return CallsResource(self._client)

    @cached_property
    def conferences(self) -> ConferencesResource:
        return ConferencesResource(self._client)

    @cached_property
    def recordings(self) -> RecordingsResource:
        return RecordingsResource(self._client)

    @cached_property
    def transcriptions(self) -> TranscriptionsResource:
        return TranscriptionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AccountsResourceWithStreamingResponse(self)

    def retrieve_recordings_json(
        self,
        account_sid: str,
        *,
        date_created: Union[str, datetime] | Omit = omit,
        page: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountRetrieveRecordingsJsonResponse:
        """
        Returns multiple recording resources for an account.

        Args:
          date_created: Filters recording by the creation date. Expected format is ISO8601 date or
              date-time, ie. {YYYY}-{MM}-{DD} or {YYYY}-{MM}-{DD}T{hh}:{mm}:{ss}Z. Also
              accepts inequality operators, e.g. DateCreated>=2023-05-22.

          page: The number of the page to be displayed, zero-indexed, should be used in
              conjuction with PageToken.

          page_size: The number of records to be displayed on a page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        return self._get(
            f"/texml/Accounts/{account_sid}/Recordings.json",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_created": date_created,
                        "page": page,
                        "page_size": page_size,
                    },
                    account_retrieve_recordings_json_params.AccountRetrieveRecordingsJsonParams,
                ),
            ),
            cast_to=AccountRetrieveRecordingsJsonResponse,
        )

    def retrieve_transcriptions_json(
        self,
        account_sid: str,
        *,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountRetrieveTranscriptionsJsonResponse:
        """
        Returns multiple recording transcription resources for an account.

        Args:
          page_size: The number of records to be displayed on a page

          page_token: Used to request the next page of results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        return self._get(
            f"/texml/Accounts/{account_sid}/Transcriptions.json",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_size": page_size,
                        "page_token": page_token,
                    },
                    account_retrieve_transcriptions_json_params.AccountRetrieveTranscriptionsJsonParams,
                ),
            ),
            cast_to=AccountRetrieveTranscriptionsJsonResponse,
        )


class AsyncAccountsResource(AsyncAPIResource):
    @cached_property
    def calls(self) -> AsyncCallsResource:
        return AsyncCallsResource(self._client)

    @cached_property
    def conferences(self) -> AsyncConferencesResource:
        return AsyncConferencesResource(self._client)

    @cached_property
    def recordings(self) -> AsyncRecordingsResource:
        return AsyncRecordingsResource(self._client)

    @cached_property
    def transcriptions(self) -> AsyncTranscriptionsResource:
        return AsyncTranscriptionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncAccountsResourceWithStreamingResponse(self)

    async def retrieve_recordings_json(
        self,
        account_sid: str,
        *,
        date_created: Union[str, datetime] | Omit = omit,
        page: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountRetrieveRecordingsJsonResponse:
        """
        Returns multiple recording resources for an account.

        Args:
          date_created: Filters recording by the creation date. Expected format is ISO8601 date or
              date-time, ie. {YYYY}-{MM}-{DD} or {YYYY}-{MM}-{DD}T{hh}:{mm}:{ss}Z. Also
              accepts inequality operators, e.g. DateCreated>=2023-05-22.

          page: The number of the page to be displayed, zero-indexed, should be used in
              conjuction with PageToken.

          page_size: The number of records to be displayed on a page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        return await self._get(
            f"/texml/Accounts/{account_sid}/Recordings.json",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "date_created": date_created,
                        "page": page,
                        "page_size": page_size,
                    },
                    account_retrieve_recordings_json_params.AccountRetrieveRecordingsJsonParams,
                ),
            ),
            cast_to=AccountRetrieveRecordingsJsonResponse,
        )

    async def retrieve_transcriptions_json(
        self,
        account_sid: str,
        *,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountRetrieveTranscriptionsJsonResponse:
        """
        Returns multiple recording transcription resources for an account.

        Args:
          page_size: The number of records to be displayed on a page

          page_token: Used to request the next page of results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        return await self._get(
            f"/texml/Accounts/{account_sid}/Transcriptions.json",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page_size": page_size,
                        "page_token": page_token,
                    },
                    account_retrieve_transcriptions_json_params.AccountRetrieveTranscriptionsJsonParams,
                ),
            ),
            cast_to=AccountRetrieveTranscriptionsJsonResponse,
        )


class AccountsResourceWithRawResponse:
    def __init__(self, accounts: AccountsResource) -> None:
        self._accounts = accounts

        self.retrieve_recordings_json = to_raw_response_wrapper(
            accounts.retrieve_recordings_json,
        )
        self.retrieve_transcriptions_json = to_raw_response_wrapper(
            accounts.retrieve_transcriptions_json,
        )

    @cached_property
    def calls(self) -> CallsResourceWithRawResponse:
        return CallsResourceWithRawResponse(self._accounts.calls)

    @cached_property
    def conferences(self) -> ConferencesResourceWithRawResponse:
        return ConferencesResourceWithRawResponse(self._accounts.conferences)

    @cached_property
    def recordings(self) -> RecordingsResourceWithRawResponse:
        return RecordingsResourceWithRawResponse(self._accounts.recordings)

    @cached_property
    def transcriptions(self) -> TranscriptionsResourceWithRawResponse:
        return TranscriptionsResourceWithRawResponse(self._accounts.transcriptions)


class AsyncAccountsResourceWithRawResponse:
    def __init__(self, accounts: AsyncAccountsResource) -> None:
        self._accounts = accounts

        self.retrieve_recordings_json = async_to_raw_response_wrapper(
            accounts.retrieve_recordings_json,
        )
        self.retrieve_transcriptions_json = async_to_raw_response_wrapper(
            accounts.retrieve_transcriptions_json,
        )

    @cached_property
    def calls(self) -> AsyncCallsResourceWithRawResponse:
        return AsyncCallsResourceWithRawResponse(self._accounts.calls)

    @cached_property
    def conferences(self) -> AsyncConferencesResourceWithRawResponse:
        return AsyncConferencesResourceWithRawResponse(self._accounts.conferences)

    @cached_property
    def recordings(self) -> AsyncRecordingsResourceWithRawResponse:
        return AsyncRecordingsResourceWithRawResponse(self._accounts.recordings)

    @cached_property
    def transcriptions(self) -> AsyncTranscriptionsResourceWithRawResponse:
        return AsyncTranscriptionsResourceWithRawResponse(self._accounts.transcriptions)


class AccountsResourceWithStreamingResponse:
    def __init__(self, accounts: AccountsResource) -> None:
        self._accounts = accounts

        self.retrieve_recordings_json = to_streamed_response_wrapper(
            accounts.retrieve_recordings_json,
        )
        self.retrieve_transcriptions_json = to_streamed_response_wrapper(
            accounts.retrieve_transcriptions_json,
        )

    @cached_property
    def calls(self) -> CallsResourceWithStreamingResponse:
        return CallsResourceWithStreamingResponse(self._accounts.calls)

    @cached_property
    def conferences(self) -> ConferencesResourceWithStreamingResponse:
        return ConferencesResourceWithStreamingResponse(self._accounts.conferences)

    @cached_property
    def recordings(self) -> RecordingsResourceWithStreamingResponse:
        return RecordingsResourceWithStreamingResponse(self._accounts.recordings)

    @cached_property
    def transcriptions(self) -> TranscriptionsResourceWithStreamingResponse:
        return TranscriptionsResourceWithStreamingResponse(self._accounts.transcriptions)


class AsyncAccountsResourceWithStreamingResponse:
    def __init__(self, accounts: AsyncAccountsResource) -> None:
        self._accounts = accounts

        self.retrieve_recordings_json = async_to_streamed_response_wrapper(
            accounts.retrieve_recordings_json,
        )
        self.retrieve_transcriptions_json = async_to_streamed_response_wrapper(
            accounts.retrieve_transcriptions_json,
        )

    @cached_property
    def calls(self) -> AsyncCallsResourceWithStreamingResponse:
        return AsyncCallsResourceWithStreamingResponse(self._accounts.calls)

    @cached_property
    def conferences(self) -> AsyncConferencesResourceWithStreamingResponse:
        return AsyncConferencesResourceWithStreamingResponse(self._accounts.conferences)

    @cached_property
    def recordings(self) -> AsyncRecordingsResourceWithStreamingResponse:
        return AsyncRecordingsResourceWithStreamingResponse(self._accounts.recordings)

    @cached_property
    def transcriptions(self) -> AsyncTranscriptionsResourceWithStreamingResponse:
        return AsyncTranscriptionsResourceWithStreamingResponse(self._accounts.transcriptions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ....._models import BaseModel

__all__ = ["CdrUsageReportResponseLegacy"]


class CdrUsageReportResponseLegacy(BaseModel):
    id: Optional[str] = None
    """Identifies the resource"""

    aggregation_type: Optional[int] = None
    """
    Aggregation type: All = 0, By Connections = 1, By Tags = 2, By Billing Group = 3
    """

    connections: Optional[List[int]] = None

    created_at: Optional[datetime] = None

    end_time: Optional[datetime] = None

    product_breakdown: Optional[int] = None
    """
    Product breakdown type: No breakdown = 0, DID vs Toll-free = 1, Country = 2, DID
    vs Toll-free per Country = 3
    """

    record_type: Optional[str] = None

    report_url: Optional[str] = None

    result: Optional[object] = None

    start_time: Optional[datetime] = None

    status: Optional[int] = None
    """Status of the report: Pending = 1, Complete = 2, Failed = 3, Expired = 4"""

    updated_at: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["DocumentListParams", "Filter", "FilterCreatedAt", "FilterCustomerReference", "FilterFilename", "Page"]


class DocumentListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter for documents (deepObject style).

    Originally: filter[filename][contains], filter[customer_reference][eq],
    filter[customer_reference][in][], filter[created_at][gt], filter[created_at][lt]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """

    sort: List[Literal["filename", "created_at", "updated_at", "-filename", "-created_at", "-updated_at"]]
    """Consolidated sort parameter for documents (deepObject style).

    Originally: sort[]
    """


class FilterCreatedAt(TypedDict, total=False):
    gt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter by created at greater than provided value."""

    lt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter by created at less than provided value."""


_FilterCustomerReferenceReservedKeywords = TypedDict(
    "_FilterCustomerReferenceReservedKeywords",
    {
        "in": SequenceNotStr[str],
    },
    total=False,
)


class FilterCustomerReference(_FilterCustomerReferenceReservedKeywords, total=False):
    eq: str
    """Filter documents by a customer reference."""


class FilterFilename(TypedDict, total=False):
    contains: str
    """Filter by string matching part of filename."""


class Filter(TypedDict, total=False):
    created_at: FilterCreatedAt

    customer_reference: FilterCustomerReference

    filename: FilterFilename


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from ..._types import SequenceNotStr

__all__ = ["ActionStopParams"]


class ActionStopParams(TypedDict, total=False):
    call_control_ids: SequenceNotStr[str]
    """
    List of call control ids identifying participants the audio file should stop be
    played to. If not given, the audio will be stoped to the entire conference.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["EventListParams", "Filter", "FilterCreatedAt", "Page"]


class EventListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[type], filter[porting_order_id], filter[created_at][gte],
    filter[created_at][lte]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """


class FilterCreatedAt(TypedDict, total=False):
    gte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter by created at greater than or equal to."""

    lte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter by created at less than or equal to."""


class Filter(TypedDict, total=False):
    created_at: FilterCreatedAt
    """Created at date range filtering operations"""

    porting_order_id: str
    """Filter by porting order ID."""

    type: Literal[
        "porting_order.deleted",
        "porting_order.loa_updated",
        "porting_order.messaging_changed",
        "porting_order.status_changed",
        "porting_order.sharing_token_expired",
        "porting_order.new_comment",
        "porting_order.split",
    ]
    """Filter by event type."""


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["ExternalVettingImportResponse"]


class ExternalVettingImportResponse(BaseModel):
    create_date: Optional[str] = FieldInfo(alias="createDate", default=None)
    """Vetting submission date.

    This is the date when the vetting request is generated in ISO 8601 format.
    """

    evp_id: Optional[str] = FieldInfo(alias="evpId", default=None)
    """External vetting provider ID for the brand."""

    vetted_date: Optional[str] = FieldInfo(alias="vettedDate", default=None)
    """Vetting effective date.

    This is the date when vetting was completed, or the starting effective date in
    ISO 8601 format. If this date is missing, then the vetting was not complete or
    not valid.
    """

    vetting_class: Optional[str] = FieldInfo(alias="vettingClass", default=None)
    """Identifies the vetting classification."""

    vetting_id: Optional[str] = FieldInfo(alias="vettingId", default=None)
    """Unique ID that identifies a vetting transaction performed by a vetting provider.

    This ID is provided by the vetting provider at time of vetting.
    """

    vetting_score: Optional[int] = FieldInfo(alias="vettingScore", default=None)
    """Vetting score ranging from 0-100."""

    vetting_token: Optional[str] = FieldInfo(alias="vettingToken", default=None)
    """Required by some providers for vetting record confirmation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date
from typing_extensions import Literal

import httpx

from ..types import charges_breakdown_retrieve_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.charges_breakdown_retrieve_response import ChargesBreakdownRetrieveResponse

__all__ = ["ChargesBreakdownResource", "AsyncChargesBreakdownResource"]


class ChargesBreakdownResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ChargesBreakdownResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ChargesBreakdownResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChargesBreakdownResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ChargesBreakdownResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        start_date: Union[str, date],
        end_date: Union[str, date] | Omit = omit,
        format: Literal["json", "csv"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargesBreakdownRetrieveResponse:
        """
        Retrieve a detailed breakdown of monthly charges for phone numbers in a
        specified date range. The date range cannot exceed 31 days.

        Args:
          start_date: Start date for the charges breakdown in ISO date format (YYYY-MM-DD)

          end_date: End date for the charges breakdown in ISO date format (YYYY-MM-DD). If not
              provided, defaults to start_date + 1 month. The date is exclusive, data for the
              end_date itself is not included in the report. The interval between start_date
              and end_date cannot exceed 31 days.

          format: Response format

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/charges_breakdown",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "start_date": start_date,
                        "end_date": end_date,
                        "format": format,
                    },
                    charges_breakdown_retrieve_params.ChargesBreakdownRetrieveParams,
                ),
            ),
            cast_to=ChargesBreakdownRetrieveResponse,
        )


class AsyncChargesBreakdownResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncChargesBreakdownResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncChargesBreakdownResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChargesBreakdownResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncChargesBreakdownResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        start_date: Union[str, date],
        end_date: Union[str, date] | Omit = omit,
        format: Literal["json", "csv"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargesBreakdownRetrieveResponse:
        """
        Retrieve a detailed breakdown of monthly charges for phone numbers in a
        specified date range. The date range cannot exceed 31 days.

        Args:
          start_date: Start date for the charges breakdown in ISO date format (YYYY-MM-DD)

          end_date: End date for the charges breakdown in ISO date format (YYYY-MM-DD). If not
              provided, defaults to start_date + 1 month. The date is exclusive, data for the
              end_date itself is not included in the report. The interval between start_date
              and end_date cannot exceed 31 days.

          format: Response format

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/charges_breakdown",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "start_date": start_date,
                        "end_date": end_date,
                        "format": format,
                    },
                    charges_breakdown_retrieve_params.ChargesBreakdownRetrieveParams,
                ),
            ),
            cast_to=ChargesBreakdownRetrieveResponse,
        )


class ChargesBreakdownResourceWithRawResponse:
    def __init__(self, charges_breakdown: ChargesBreakdownResource) -> None:
        self._charges_breakdown = charges_breakdown

        self.retrieve = to_raw_response_wrapper(
            charges_breakdown.retrieve,
        )


class AsyncChargesBreakdownResourceWithRawResponse:
    def __init__(self, charges_breakdown: AsyncChargesBreakdownResource) -> None:
        self._charges_breakdown = charges_breakdown

        self.retrieve = async_to_raw_response_wrapper(
            charges_breakdown.retrieve,
        )


class ChargesBreakdownResourceWithStreamingResponse:
    def __init__(self, charges_breakdown: ChargesBreakdownResource) -> None:
        self._charges_breakdown = charges_breakdown

        self.retrieve = to_streamed_response_wrapper(
            charges_breakdown.retrieve,
        )


class AsyncChargesBreakdownResourceWithStreamingResponse:
    def __init__(self, charges_breakdown: AsyncChargesBreakdownResource) -> None:
        self._charges_breakdown = charges_breakdown

        self.retrieve = async_to_streamed_response_wrapper(
            charges_breakdown.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .pagination_meta import PaginationMeta

__all__ = ["MobileNetworkOperatorListResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    country_code: Optional[str] = None
    """The mobile operator two-character (ISO 3166-1 alpha-2) origin country code."""

    mcc: Optional[str] = None
    """MCC stands for Mobile Country Code.

    It's a three decimal digit that identifies a country.<br/><br/> This code is
    commonly seen joined with a Mobile Network Code (MNC) in a tuple that allows
    identifying a carrier known as PLMN (Public Land Mobile Network) code.
    """

    mnc: Optional[str] = None
    """MNC stands for Mobile Network Code.

    It's a two to three decimal digits that identify a network.<br/><br/> This code
    is commonly seen joined with a Mobile Country Code (MCC) in a tuple that allows
    identifying a carrier known as PLMN (Public Land Mobile Network) code.
    """

    name: Optional[str] = None
    """The network operator name."""

    network_preferences_enabled: Optional[bool] = None
    """
    Indicate whether the mobile network operator can be set as preferred in the
    Network Preferences API.
    """

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    tadig: Optional[str] = None
    """TADIG stands for Transferred Account Data Interchange Group.

    The TADIG code is a unique identifier for network operators in GSM mobile
    networks.
    """


class MobileNetworkOperatorListResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    FaxApplicationListResponse,
    FaxApplicationCreateResponse,
    FaxApplicationDeleteResponse,
    FaxApplicationUpdateResponse,
    FaxApplicationRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestFaxApplications:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        fax_application = client.fax_applications.create(
            application_name="fax-router",
            webhook_event_url="https://example.com",
        )
        assert_matches_type(FaxApplicationCreateResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        fax_application = client.fax_applications.create(
            application_name="fax-router",
            webhook_event_url="https://example.com",
            active=False,
            anchorsite_override="Amsterdam, Netherlands",
            inbound={
                "channel_limit": 10,
                "sip_subdomain": "example",
                "sip_subdomain_receive_settings": "only_my_connections",
            },
            outbound={
                "channel_limit": 10,
                "outbound_voice_profile_id": "1293384261075731499",
            },
            tags=["tag1", "tag2"],
            webhook_event_failover_url="https://failover.example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(FaxApplicationCreateResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.fax_applications.with_raw_response.create(
            application_name="fax-router",
            webhook_event_url="https://example.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax_application = response.parse()
        assert_matches_type(FaxApplicationCreateResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.fax_applications.with_streaming_response.create(
            application_name="fax-router",
            webhook_event_url="https://example.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax_application = response.parse()
            assert_matches_type(FaxApplicationCreateResponse, fax_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        fax_application = client.fax_applications.retrieve(
            "1293384261075731499",
        )
        assert_matches_type(FaxApplicationRetrieveResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.fax_applications.with_raw_response.retrieve(
            "1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax_application = response.parse()
        assert_matches_type(FaxApplicationRetrieveResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.fax_applications.with_streaming_response.retrieve(
            "1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax_application = response.parse()
            assert_matches_type(FaxApplicationRetrieveResponse, fax_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.fax_applications.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        fax_application = client.fax_applications.update(
            id="1293384261075731499",
            application_name="fax-router",
            webhook_event_url="https://example.com",
        )
        assert_matches_type(FaxApplicationUpdateResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        fax_application = client.fax_applications.update(
            id="1293384261075731499",
            application_name="fax-router",
            webhook_event_url="https://example.com",
            active=False,
            anchorsite_override="Amsterdam, Netherlands",
            fax_email_recipient="user@example.com",
            inbound={
                "channel_limit": 10,
                "sip_subdomain": "example",
                "sip_subdomain_receive_settings": "only_my_connections",
            },
            outbound={
                "channel_limit": 10,
                "outbound_voice_profile_id": "1293384261075731499",
            },
            tags=["tag1", "tag2"],
            webhook_event_failover_url="https://failover.example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(FaxApplicationUpdateResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.fax_applications.with_raw_response.update(
            id="1293384261075731499",
            application_name="fax-router",
            webhook_event_url="https://example.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax_application = response.parse()
        assert_matches_type(FaxApplicationUpdateResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.fax_applications.with_streaming_response.update(
            id="1293384261075731499",
            application_name="fax-router",
            webhook_event_url="https://example.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax_application = response.parse()
            assert_matches_type(FaxApplicationUpdateResponse, fax_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.fax_applications.with_raw_response.update(
                id="",
                application_name="fax-router",
                webhook_event_url="https://example.com",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        fax_application = client.fax_applications.list()
        assert_matches_type(FaxApplicationListResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        fax_application = client.fax_applications.list(
            filter={
                "application_name": {"contains": "fax-app"},
                "outbound_voice_profile_id": "1293384261075731499",
            },
            page={
                "number": 1,
                "size": 1,
            },
            sort="application_name",
        )
        assert_matches_type(FaxApplicationListResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.fax_applications.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax_application = response.parse()
        assert_matches_type(FaxApplicationListResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.fax_applications.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax_application = response.parse()
            assert_matches_type(FaxApplicationListResponse, fax_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        fax_application = client.fax_applications.delete(
            "1293384261075731499",
        )
        assert_matches_type(FaxApplicationDeleteResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.fax_applications.with_raw_response.delete(
            "1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax_application = response.parse()
        assert_matches_type(FaxApplicationDeleteResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.fax_applications.with_streaming_response.delete(
            "1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax_application = response.parse()
            assert_matches_type(FaxApplicationDeleteResponse, fax_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.fax_applications.with_raw_response.delete(
                "",
            )


class TestAsyncFaxApplications:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        fax_application = await async_client.fax_applications.create(
            application_name="fax-router",
            webhook_event_url="https://example.com",
        )
        assert_matches_type(FaxApplicationCreateResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        fax_application = await async_client.fax_applications.create(
            application_name="fax-router",
            webhook_event_url="https://example.com",
            active=False,
            anchorsite_override="Amsterdam, Netherlands",
            inbound={
                "channel_limit": 10,
                "sip_subdomain": "example",
                "sip_subdomain_receive_settings": "only_my_connections",
            },
            outbound={
                "channel_limit": 10,
                "outbound_voice_profile_id": "1293384261075731499",
            },
            tags=["tag1", "tag2"],
            webhook_event_failover_url="https://failover.example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(FaxApplicationCreateResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.fax_applications.with_raw_response.create(
            application_name="fax-router",
            webhook_event_url="https://example.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax_application = await response.parse()
        assert_matches_type(FaxApplicationCreateResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.fax_applications.with_streaming_response.create(
            application_name="fax-router",
            webhook_event_url="https://example.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax_application = await response.parse()
            assert_matches_type(FaxApplicationCreateResponse, fax_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        fax_application = await async_client.fax_applications.retrieve(
            "1293384261075731499",
        )
        assert_matches_type(FaxApplicationRetrieveResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.fax_applications.with_raw_response.retrieve(
            "1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax_application = await response.parse()
        assert_matches_type(FaxApplicationRetrieveResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.fax_applications.with_streaming_response.retrieve(
            "1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax_application = await response.parse()
            assert_matches_type(FaxApplicationRetrieveResponse, fax_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.fax_applications.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        fax_application = await async_client.fax_applications.update(
            id="1293384261075731499",
            application_name="fax-router",
            webhook_event_url="https://example.com",
        )
        assert_matches_type(FaxApplicationUpdateResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        fax_application = await async_client.fax_applications.update(
            id="1293384261075731499",
            application_name="fax-router",
            webhook_event_url="https://example.com",
            active=False,
            anchorsite_override="Amsterdam, Netherlands",
            fax_email_recipient="user@example.com",
            inbound={
                "channel_limit": 10,
                "sip_subdomain": "example",
                "sip_subdomain_receive_settings": "only_my_connections",
            },
            outbound={
                "channel_limit": 10,
                "outbound_voice_profile_id": "1293384261075731499",
            },
            tags=["tag1", "tag2"],
            webhook_event_failover_url="https://failover.example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(FaxApplicationUpdateResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.fax_applications.with_raw_response.update(
            id="1293384261075731499",
            application_name="fax-router",
            webhook_event_url="https://example.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax_application = await response.parse()
        assert_matches_type(FaxApplicationUpdateResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.fax_applications.with_streaming_response.update(
            id="1293384261075731499",
            application_name="fax-router",
            webhook_event_url="https://example.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax_application = await response.parse()
            assert_matches_type(FaxApplicationUpdateResponse, fax_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.fax_applications.with_raw_response.update(
                id="",
                application_name="fax-router",
                webhook_event_url="https://example.com",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        fax_application = await async_client.fax_applications.list()
        assert_matches_type(FaxApplicationListResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        fax_application = await async_client.fax_applications.list(
            filter={
                "application_name": {"contains": "fax-app"},
                "outbound_voice_profile_id": "1293384261075731499",
            },
            page={
                "number": 1,
                "size": 1,
            },
            sort="application_name",
        )
        assert_matches_type(FaxApplicationListResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.fax_applications.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax_application = await response.parse()
        assert_matches_type(FaxApplicationListResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.fax_applications.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax_application = await response.parse()
            assert_matches_type(FaxApplicationListResponse, fax_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        fax_application = await async_client.fax_applications.delete(
            "1293384261075731499",
        )
        assert_matches_type(FaxApplicationDeleteResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.fax_applications.with_raw_response.delete(
            "1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax_application = await response.parse()
        assert_matches_type(FaxApplicationDeleteResponse, fax_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.fax_applications.with_streaming_response.delete(
            "1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax_application = await response.parse()
            assert_matches_type(FaxApplicationDeleteResponse, fax_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.fax_applications.with_raw_response.delete(
                "",
            )
